<?php
/*********************************************************************\
| (c)2017 codemonster                                                 |
|---------------------------------------------------------------------|
| This program is free software; you can redistribute it and/or       |
| modify it under the terms of the GNU General Public License         |
| as published by the Free Software Foundation; either version 2      |
| of the License, or (at your option) any later version.              |
|                                                                     |
| This program is distributed in the hope that it will be useful,     |
| but WITHOUT ANY WARRANTY; without even the implied warranty of      |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       |
| GNU General Public License for more details.                        |
|                                                                     |
| You should have received a copy of the GNU General Public License   |
| along with this program; If not, see <http://www.gnu.org/licenses/> |
\*********************************************************************/
?>
<?php
class CodemonsterDlmSearchSkyTorrents
{
		private $url_domain = 'https://www.skytorrents.in';
		private $qurl = '/?query=%s&sort=seeders';
		private $proxySource = 'http://www.dlmapi.codemonster.be/v1/extensionsources/extension/5/%s';
		
        private $tempFile = '/tmp/_DLM_SkyTorrents.log';
		private $debug  = false;
		private $preparedUrl = '';
       
        public function __construct() 
		{
			if(file_exists($this->tempFile))
			{
				unlink($this->tempFile);
			}
			
			if ($this->debug) 
			{
				ini_set('display_errors', 0);
				ini_set('log_errors', 1);
				ini_set('error_log', $this->tempFile);
			}
        }

        public function prepare($curl, $query) 
		{
			try
			{				
				$proxyUrl = $this->getProxyUrl(sprintf($this->proxySource, urlencode(str_replace(' ', '_', $query))));
				
				if(!empty($proxyUrl))
				{
					$this->url_domain = $proxyUrl;
				}
				
				$this->qurl=$this->url_domain.$this->qurl;
				$this->preparedUrl = sprintf($this->qurl, urlencode(str_replace(' ', '+', $query)));
				$this->configureCurl($curl, $this->preparedUrl);  
			} catch (Exception $e) 
			{
				$this->DebugLog($e);
			}
        }

        public function parse($plugin, $response)
		{
			//$this->DebugLog("page: 1");
			$this->processResultList($plugin, $response);
			try
			{
				for ($i = 2; $i <= 4; $i++) // 4 pages, starting from the second page -> 200 results
				{
					//$this->DebugLog("page: ".$i);
					
					$response = $this->executeCurl($this->preparedUrl.'&page=');
					
					$this->DebugLog('Response proxy url : '.$response);
					
					$output = preg_replace_callback("/(&#[0-9]+;)/", function($m) { return mb_convert_encoding($m[1], "UTF-8", "HTML-ENTITIES"); }, $response);

					$this->processResultList($plugin, $output);
				}
			} 
			catch (Exception $e) 
			{
				$this->DebugLog($e);
			}
        }
		
		/* Begin private methods */
		
		private function DebugLog($str) 
		{
			if ($this->debug) 
			{
				file_put_contents($this->tempFile,date('Y-m-d H:i:s').' | '.$str."\n",FILE_APPEND);
			}
		}
		
		private function configureCurl($curl, $url)
		{
			$headers = array
				(
					'Accept: application/json,text/html,application/xhtml+xml,application/xml;q=0.9,*;q=0.8',
					'Accept-Language: en-us;q=0.7,en;q=0.3',
					'Accept-Encoding: deflate',
					'Accept-Charset: windows-1251,utf-8;q=0.7,*;q=0.7'
				);
				
				curl_setopt($curl, CURLOPT_HTTPHEADER,$headers); 
				curl_setopt($curl, CURLOPT_URL, $url);
				curl_setopt($curl, CURLOPT_FAILONERROR, 1);
				curl_setopt($curl, CURLOPT_FOLLOWLOCATION, 1);
				curl_setopt($curl, CURLOPT_RETURNTRANSFER,1);
				curl_setopt($curl, CURLOPT_TIMEOUT, 240);
				curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 5);//Tell cURL that it should only spend 5 seconds trying to connect to the URL.
				curl_setopt($curl, CURLOPT_USERAGENT, 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/66.0.3359.139 Safari/537.36');
				curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
				curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false); 
		}
		
		private function executeCurl($url)
		{
			$curl = curl_init();
			$this->configureCurl($curl, $url);
			$content = curl_exec($curl);
			curl_close($curl);
			return $content;
		}
		
		private function getProxyUrl($proxySource)
		{
			$this->DebugLog('Proxy source : '.$proxySource);
			$response = $this->executeCurl($proxySource);
			$this->DebugLog('Result proxy urls : '.$response);
			
			$jsonResult = json_decode($response, true);
			if(isset($jsonResult))		
			{
				foreach($jsonResult as $result)
				{
					foreach($result as $source)
					{
						$url = rtrim($source, '/');
						$queryUrl = $url.$this->qurl;
						$this->DebugLog($url);
				
						$responseProxy = $this->executeCurl(sprintf($queryUrl, urlencode('h'))); // just search for some random stuff
						$regexp_magnet = 'href="(magnet.*)"';
						if(preg_match_all("/$regexp_magnet/siU", $responseProxy, $magnets, PREG_SET_ORDER))
						{
							$this->DebugLog('Proxy url to use : '.$url);
							return $url;
						}
						
						$this->DebugLog('Proxy response : '.$responseProxy);
					}
				}
			}
			return null;
		}
		
		private function processResultList($plugin, $response)
		{
			$regex_Items = '(<tr class=\".*\">.*magnet.*<\/tr>)';
			
			if(preg_match_all("/$regex_Items/Us", $response, $items, PREG_SET_ORDER)) 
			{
				foreach ($items as $item)
				{
					$recordToAdd = $this->createItem($item[1]);
					$this->addItemToPlugin($plugin, $recordToAdd);	
				}
			}
		}
		
		private function createItem($output)
		{
			//$this->DebugLog('Item to process : '.$output);
			
			$regex_properties = '<td.*>.*href="(.*)".*>(.*)<\/a>.*href="(magnet.*btih:(.*)&.*)".*<\/td>.*<td.*>([0-9\.]+) (GB|MB|KB)<\/td>.*<td.*>.*<\/td>.*<td.*>(.*)<\/td>.*<td.*>([0-9\.]+)<\/td>.*<td.*>([0-9\.]+)<\/td>';
			//$regex_title = '<a href="(\/files\/details.*)">(.*)<\/a>'; -> 2
			//$regex_magnet = 'href="(magnet.*)"'; -> 3
			//$regex_hash = 'btih:(.*)&'; -> 4
			//$regex_url = 'btih:(.*)&'; -> 1
			//$regex_category = '... Unknown ...';
			//$regex_dateTime = 'btih:(.*)&'; -> 6
			//$regex_size = 'align=\"right\">(.*) (GB|MB|KB)<'; -> 5
			//$regex_seeders = 'green">(.*)<\/f'; -> 7
			//$regex_leechers = 'red">(.*)<\/f'; -> 8

			$recordToAdd = array();
						
			if(preg_match_all("/$regex_properties/Us", $output, $fields, PREG_SET_ORDER))
			{
				foreach ($fields as $field)
				{
					$recordToAdd["page"] = $this->url_domain.$field[1]; //1. url
					$recordToAdd["seeds"] = $field[8]; //2. seeds
					$recordToAdd["leechs"] = $field[9]; //3. leechs
					$recordToAdd["datetime"] = $this->getDate($field[7]); //4. date
					$recordToAdd["title"] = $this->getTitle($field[2]); //5. title
					$recordToAdd["download"] = $field[3]; //6. magnet
					$recordToAdd["category"] = "Unknown"; //7. category
					$recordToAdd["size"] = $this->getSize(str_replace(",",".",$field[5]), $field[6]); //8. size
					$recordToAdd["hash"] = $field[4]; //9. hash
				}
			}

			return $recordToAdd;
		}
		
		private function getTitle($title)
		{
			$title = preg_replace_callback("/(&#[0-9]+;)/", function($m) { return mb_convert_encoding($m[1], "UTF-8", "HTML-ENTITIES"); }, $title); 
			$title = htmlspecialchars_decode($title);
			return $title;
		}
		
		private function getDate($dateString)
		{
			if (strpos($dateString, 'ago') !== false) 
			{
				$dateString = trim(str_replace("ago", "", $dateString));
				$date = date('Y-m-d',strtotime("-".$dateString));
				return $date;
			}
			
			$date = DateTime::createFromFormat('d M Y', $dateString)->format('Y-m-d  H:i:s');
			// 12-21-2016
			return $date;
		}
		
		private function getSize($size, $size_dim)
		{
			switch ($size_dim)
			{
				 case 'KB':
					 return $size * 1024;
				 case 'MB':
					 return $size * 1024 * 1024;
				 case 'GB': 
					 return $size * 1024 * 1024 * 1024;
			}
			return $size;
		}
		
		private function addItemToPlugin($plugin, $recordToAdd)
		{
			if (array_key_exists('title', $recordToAdd) && strlen($recordToAdd["title"]) > 0) 
			{
				$plugin->addResult($recordToAdd["title"],
					$recordToAdd["download"],
					(float)$recordToAdd["size"],
					date('Y-m-d',strtotime(str_replace("'", "", $recordToAdd["datetime"]))),
					$recordToAdd["page"],
					$recordToAdd["hash"],
					(int)$recordToAdd["seeds"],
					(int)$recordToAdd["leechs"],
					$recordToAdd["category"]);
			}
		}
				
		/* End private methods */
}
?>